def dict_compare(old_dict: dict, new_dict: dict):
    old_keys = set(old_dict.keys())
    new_keys = set(new_dict.keys())
    intersect = old_keys & new_keys
    added = new_keys - old_keys
    removed = old_keys - new_keys
    modified = {o: (old_dict[o], new_dict[o]) for o in intersect if old_dict[o] != new_dict[o]}
    same = set(o for o in intersect if old_dict[o] == new_dict[o] or (old_dict[o] is None and new_dict[o] is None))
    return {'added': list(added), 'removed': list(removed), 'modified': modified, 'same': list(same)}

def subset_dict(dict1: dict, dict2: dict):
    return set(dict1.items()) <= set(dict2.items())

def dict_find_keys(list_dict: list, filter: dict, multiple=True):
    result = [x for x in list_dict if subset_dict(filter, x)]
    if not multiple:
        return result[0] if result else {}
    return result



import os
import re
import pathlib

def get_file_extension(filename):
    return ''.join(pathlib.Path(filename).suffixes)

def get_list_files(path: str, filter=None, filter_not_contain: str = None, add_path: bool = False, path2: str = ''):
    path = os.path.normpath(path)
    list_files = []
    if add_path:
        path2 = path2 or path
        path2 = os.path.normpath(path2)
    else:
        path2 = ''
    if filter_not_contain:
        filter = r'^((?!{filter_not_contain}).)*$'.format(filter_not_contain=filter_not_contain)
    if filter:
        filters = filter if isinstance(filter, list) else [filter]
        for f in filters:
            list_files.extend([os.path.join(path2, x) for x in os.listdir(path) if re.search(f, x)])
    else:
        list_files = [os.path.join(path2, x) for x in os.listdir(path)]
    return list(set(list_files))

def get_list_files_recursive(path: str, filter=None, filter_not_contain: str = None, add_path: bool = False, path2: str = ''):
    """
    Get filelist with path in folder
    :param path: Folder containing files
    :param filter: filter (regexp-like), could be list of str or just str. None by default
    :param filter_not_contain: filter which should not be in filenames (regexp-like)
    :param add_path: bool, will needed add path to file or not
    :param path2: path which need to add
    :return: return list of files with/without path to it
    """
    path = os.path.normpath(path)
    list_files = []
    if add_path:
        if path2 == '':
            path2 = path
        else:
            path2 = os.path.normpath(path2)
    else:
        path2 = ''

    if filter_not_contain:
        filter = r'^((?!{filter_not_contain}).)*$'.format(filter_not_contain=filter_not_contain)

    if filter:
        if type(filter) == list:
            for f in filter:
                for path_, subdirs, files in os.walk(path):
                    list_files.extend([os.path.join(path_, file) for file in files if re.search(f, file)])
        if type(filter) == str:
            for path_, subdirs, files in os.walk(path):
                list_files.extend([os.path.join(path_, file) for file in files if re.search(filter, file)])
    else:
        for path_, subdirs, files in os.walk(path):
            list_files.extend([os.path.join(path_, x) for x in files])

    list_files_ = []
    for file in list_files:
        path_ = os.path.split(file)
        path_ = [os.path.split(path_[0])[1], path_[1]]
        if add_path:
            path_ = [path2, *path_]
        list_files_.append(os.path.join(*path_))

    list_files = list(set(list_files_))

    return list_files

def check_folder_exist(folder_path: str):
    folder_path = os.path.normpath(folder_path)
    if not os.path.exists(folder_path):
        os.makedirs(folder_path)

ls = get_list_files
lsr = get_list_files_recursive



import sys
import os

def get_args():
    args = sys.argv
    args_d = {}
    for i, arg in enumerate(args):
        if i == 0:
            args_d['name'] = arg
            continue
        if arg.startswith('-') or arg.startswith('--'):
            arg = arg.lstrip('-')
            if i != len(args) - 1 and not (args[i + 1].startswith('-') or args[i + 1].startswith('--')):
                v = args[i + 1]
                for cast in (int, float, str):
                    try:
                        v = cast(v)
                        break
                    except ValueError:
                        continue
                args_d[arg] = v
            else:
                args_d[arg] = True
    return args_d

def get_log_param(args: dict):
    log_level = args.get('log_level') or args.get('ll') or 'INFO'
    log_name = args.get('log_name') or args.get('ln')
    log_dir = args.get('log_dir') or args.get('ld') or os.path.join(get_base_dir(), 'logs')
    return {'log_level': log_level, 'log_name': log_name, 'log_dir': log_dir}

def get_base_dir():
    return os.getcwd()



import os
import json
import yaml
from datetime import datetime, date
from typing import Optional, Any, Union

from .fsutils import check_folder_exist
from .stringutils import varName
from .dateutils import parse_date


class BaseMl:
    def __init__(self, fname: Optional[str] = None, encoding: str = 'utf-8', return_: Any = None):
        if fname:
            file_dir, file_name = os.path.split(fname)
        else:
            file_dir, file_name = None, None
        self.fname = file_name
        self.fpath = file_dir
        self.encoding = encoding
        self.return_ = return_
        self.data = None

    def reinit(self, fname: Optional[str] = None, encoding: str = 'utf-8', return_: Any = None):
        if fname:
            file_dir, file_name = os.path.split(fname)
        else:
            file_dir, file_name = None, None
        self.fname = file_name
        self.fpath = file_dir
        self.encoding = encoding
        self.return_ = return_

    def set_data(self, data: Any):
        self.data = data

    def get_data(self):
        return self.data


class Yaml(BaseMl):
    def parse(self):
        try:
            with open(os.path.join(self.fpath, self.fname), 'r', encoding=self.encoding) as yaml_file:
                self.data = yaml.full_load(yaml_file)
        except Exception:
            self.data = self.return_
        return self.data

    def create(self):
        if self.fname:
            check_folder_exist(self.fpath)
            with open(os.path.join(self.fpath, self.fname), 'w', encoding=self.encoding) as outfile:
                yaml.dump(self.data, outfile, default_flow_style=False, indent=4,
                          allow_unicode=True, sort_keys=False)
        else:
            raise ValueError('Please provide filename first!')


class Json(BaseMl):
    def parse(self):
        try:
            with open(os.path.join(self.fpath, self.fname), 'r', encoding=self.encoding) as json_file:
                json_str = json_file.read()
                self.data = json.loads(json_str, object_hook=self.date_hook)
        except Exception:
            self.data = self.return_
        return self.data

    def create(self):
        if self.fname:
            check_folder_exist(self.fpath)
            dthandler = lambda obj: obj.isoformat() if isinstance(obj, (datetime, date)) else None
            with open(os.path.join(self.fpath, self.fname), 'w', encoding=self.encoding) as outfile:
                json.dump(self.data, outfile, ensure_ascii=False, default=dthandler, indent=4)
        else:
            raise ValueError('Please provide filename first!')

    def dumps(self):
        dthandler = lambda obj: obj.isoformat() if isinstance(obj, (datetime, date)) else None
        return json.dumps(self.data, ensure_ascii=False, default=dthandler, indent=4)

    @staticmethod
    def date_hook(json_dict: dict):
        for (key, value) in json_dict.items():
            date_value = parse_date(value)
            if isinstance(date_value, (date, datetime)):
                json_dict[key] = date_value
        return json_dict


def create_json(json_data: Any, json_fname: str = None, encoding: str = 'utf-8'):
    if not json_fname:
        json_fname = f'{varName(json_data)}.json'
    j = Json(json_fname, encoding)
    j.set_data(json_data)
    j.create()


def parse_json(json_fname: str, encoding: str = 'utf-8', return_: Any = None):
    j = Json(json_fname, encoding, return_)
    return j.parse()


def create_yaml(yaml_data: Any, yaml_fname: str = None, encoding: str = 'utf-8'):
    if not yaml_fname:
        yaml_fname = f'{varName(yaml_data)}.yml'
    y = Yaml(yaml_fname, encoding)
    y.set_data(yaml_data)
    y.create()


def parse_yaml(yaml_fname: str, encoding: str = 'utf-8', return_: Any = None):
    y = Yaml(yaml_fname, encoding, return_)
    return y.parse()



import pickle
from typing import Any

def pickle_data(data: Any, fname: str):
    with open(fname, 'wb') as f:
        pickle.dump(data, f)

def unpickle_data(fname: str, return_: Any = None):
    try:
        with open(fname, 'rb') as f:
            data = pickle.load(f)
    except FileNotFoundError:
        data = return_
    return data



from .dictutils import dict_compare, subset_dict, dict_find_keys
from .argutils import get_args, get_log_param, get_base_dir
from .fsutils import get_file_extension, get_list_files, get_list_files_recursive, check_folder_exist
from .dateutils import parse_date
from .sliceutils import slice_obj, slice_portion, slice_portion_2
from .stringutils import simple_translit, varName, get_com
from .jsonyaml import (
    Json, Yaml, BaseMl,
    create_json, parse_json, create_yaml, parse_yaml
)
from .pickleutils import pickle_data, unpickle_data

__all__ = [
    # dictutils
    "dict_compare", "subset_dict", "dict_find_keys",
    # argutils
    "get_args", "get_log_param", "get_base_dir",
    # fsutils
    "get_file_extension", "get_list_files", "get_list_files_recursive", "check_folder_exist",
    # dateutils
    "parse_date",
    # sliceutils
    "slice_obj", "slice_portion", "slice_portion_2",
    # stringutils
    "simple_translit", "varName", "get_com",
    # jsonyaml
    "Json", "Yaml", "BaseMl",
    "create_json", "parse_json", "create_yaml", "parse_yaml",
    # pickleutils
    "pickle_data", "unpickle_data",
]


import inspect

def simple_translit(text: str):
    symbols = (u"абвгдеёжзийклмнопрстуфхцчшщъыьэюяАБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯ",
               u"abvgdeejzijklmnoprstufhzcss_y_euaABVGDEEJZIJKLMNOPRSTUFHZCSS_Y_EUA")
    tr = {ord(a): ord(b) for a, b in zip(*symbols)}
    return text.translate(tr)

def varName(var):
    lcls = inspect.stack()[2][0].f_locals
    for name in lcls:
        if id(var) == id(lcls[name]):
            return name
    return None

def get_com(count, word_root, endings=None):
    if endings is None:
        endings = ['', 'а', 'ов']
    if count == 0:
        word = word_root + endings[2]
    else:
        inumber = count % 100
        if 11 <= inumber <= 19:
            word = word_root + endings[2]
        else:
            iinumber = inumber % 10
            if iinumber == 1:
                word = word_root + endings[0]
            elif iinumber in [2,3,4]:
                word = word_root + endings[1]
            else:
                word = word_root + endings[2]
    return f'{count} {word}'



from itertools import chain

def slice_obj(iterable_object, step):
    steps = [i for i in range(len(iterable_object)) if i % step == 0 and i > 0]
    pairs = zip(chain([0], steps), chain(steps, [None]))
    for i, j in pairs:
        yield iterable_object[i:j]

def slice_portion(iterable_obj, portion_qty):
    k, m = divmod(len(iterable_obj), portion_qty)
    return (iterable_obj[i * k + min(i, m):(i + 1) * k + min(i + 1, m)] for i in range(portion_qty))

def slice_portion_2(iterable_obj, portion_qty):
    return slice_obj(iterable_obj, len(iterable_obj) // portion_qty + 1)



import re

from dateutil import parser
from typing import Union

def parse_date(date_str: Union[bytes, str]):
    try:
        if isinstance(date_str, bytes):
            date_str = date_str.decode('utf-8')
        if re.match(r'\d{4}-\d{2}-\d{2}', date_str):
            return parser.parse(date_str, dayfirst=False)
        elif re.match(r'\d{2}[-.]\d{2}[-.]\d{4}', date_str):
            return parser.parse(date_str, dayfirst=True)
        elif re.match(r'\d{2}/\d{2}/\d{4}', date_str):
            return parser.parse(date_str, dayfirst=False)
    except Exception:
        return None
    return None